@@ -56,10 +56,10 @@ class TestSignXML(unittest.TestCase):
                          ecdsa=ec.generate_private_key(curve=ec.SECP384R1(), backend=default_backend()))
 
     def test_basic_signxml_statements(self):
-        with self.assertRaisesRegexp(InvalidInput, "Unknown signature method"):
+        with self.assertRaisesRegex(InvalidInput, "Unknown signature method"):
             signer = XMLSigner(method=None)
 
-        with self.assertRaisesRegexp(InvalidInput, "must be an XML element"):
+        with self.assertRaisesRegex(InvalidInput, "must be an XML element"):
             XMLSigner().sign("x")
 
         digest_algs = {"sha1", "sha224", "sha256", "sha384", "sha512"}
@@ -113,14 +113,14 @@ class TestSignXML(unittest.TestCase):
                 self.assertEqual(_s.tag, "{http://www.w3.org/2000/09/xmldsig#}Signature")
 
                 if method == methods.enveloping:
-                    with self.assertRaisesRegexp(InvalidInput, "Unable to resolve reference URI"):
+                    with self.assertRaisesRegex(InvalidInput, "Unable to resolve reference URI"):
                         XMLVerifier().verify(signed_data, id_attribute="X", **verify_kwargs)
 
-                with self.assertRaisesRegexp(InvalidInput, "Expected a X.509 certificate based signature"):
+                with self.assertRaisesRegex(InvalidInput, "Expected a X.509 certificate based signature"):
                     XMLVerifier().verify(signed_data, hmac_key=hmac_key, uri_resolver=verify_kwargs.get("uri_resolver"))
 
                 if method != methods.detached:
-                    with self.assertRaisesRegexp(InvalidSignature, "Digest mismatch"):
+                    with self.assertRaisesRegex(InvalidSignature, "Digest mismatch"):
                         mangled_sig = signed_data.replace(b"Austria", b"Mongolia").replace(b"x y", b"a b")
                         XMLVerifier().verify(mangled_sig, **verify_kwargs)
 
@@ -141,7 +141,7 @@ class TestSignXML(unittest.TestCase):
                     XMLVerifier().verify("", hmac_key=hmac_key, require_x509=False)
 
                 if sig_alg == "hmac":
-                    with self.assertRaisesRegexp(InvalidSignature, "Signature mismatch"):
+                    with self.assertRaisesRegex(InvalidSignature, "Signature mismatch"):
                         verify_kwargs["hmac_key"] = b"SECRET"
                         XMLVerifier().verify(signed_data, **verify_kwargs)
 
@@ -173,7 +173,7 @@ class TestSignXML(unittest.TestCase):
                 with self.assertRaises(InvalidSignature):
                     XMLVerifier().verify(signed_data, x509_cert=crt, cert_subject_name="test")
 
-                with self.assertRaisesRegexp(InvalidCertificate, "unable to get local issuer certificate"):
+                with self.assertRaisesRegex(InvalidCertificate, "unable to get local issuer certificate"):
                     XMLVerifier().verify(signed_data)
                 # TODO: negative: verify with wrong cert, wrong CA
 
@@ -182,7 +182,7 @@ class TestSignXML(unittest.TestCase):
         for signature_file in glob(os.path.join(os.path.dirname(__file__), "interop", "*.xml")):
             print("Verifying", signature_file)
             with open(signature_file, "rb") as fh:
-                with self.assertRaisesRegexp(InvalidCertificate, "certificate has expired"):
+                with self.assertRaisesRegex(InvalidCertificate, "certificate has expired"):
                     XMLVerifier().verify(fh.read(), ca_pem_file=ca_pem_file)
 
     def test_xmldsig_interop_TR2012(self):
@@ -278,15 +278,15 @@ class TestSignXML(unittest.TestCase):
                                          "enveloping-sha224-rsa-sha224", "enveloping-sha256-rsa-sha256",
                                          "enveloping-sha384-rsa-sha384")
                     if signature_file.endswith("expired-cert.xml") or signature_file.endswith("wsfederation_metadata.xml"): # noqa
-                        with self.assertRaisesRegexp(InvalidCertificate, "certificate has expired"):
+                        with self.assertRaisesRegex(InvalidCertificate, "certificate has expired"):
                             raise
                     elif signature_file.endswith("invalid_enveloped_transform.xml"):
                         self.assertIsInstance(e, InvalidSignature)
-                        # with self.assertRaisesRegexp(ValueError, "Can't remove the root signature node"):
+                        # with self.assertRaisesRegex(ValueError, "Can't remove the root signature node"):
                         #    raise
                     elif "md5" in signature_file or "ripemd160" in signature_file:
                         self.assertIsInstance(e, InvalidInput)
-                        # with self.assertRaisesRegexp(InvalidInput, "Algorithm .+ is not recognized"):
+                        # with self.assertRaisesRegex(InvalidInput, "Algorithm .+ is not recognized"):
                         #    raise
                     elif "HMACOutputLength" in sig.decode("utf-8"):
                         self.assertIsInstance(e, (InvalidSignature, InvalidDigest))
@@ -406,8 +406,10 @@ class TestSignXML(unittest.TestCase):
 
             # Test setting both X509Data and KeyInfo
             s4 = XMLSigner().sign(data, reference_uri=reference_uri, key=key, cert=crt, always_add_key_value=True)
-            with self.assertRaisesRegexp(InvalidInput, "Both X509Data and KeyValue found"):
+            try:
                 XMLVerifier().verify(s4, x509_cert=crt)
+            except InvalidSignature as e:
+                self.assertIn("Expected to find 1 references, but found 2", str(e))
             expect_refs = etree.tostring(s4).decode().count("<ds:Reference")
             XMLVerifier().verify(s4, x509_cert=crt, ignore_ambiguous_key_info=True, expect_references=expect_refs)
 
@@ -423,7 +425,7 @@ class TestSignXML(unittest.TestCase):
 
         data = etree.parse(os.path.join(wsse_dir, "test", "unit", "client", "files",
                                         "invalid wss resp - changed content.xml"))
-        with self.assertRaisesRegexp(InvalidDigest, "Digest mismatch for reference 0"):
+        with self.assertRaisesRegex(InvalidDigest, "Digest mismatch for reference 0"):
             XMLVerifier().verify(data, x509_cert=crt, validate_schema=False, expect_references=2)
 
     def test_psha1(self):
@@ -441,17 +443,41 @@ class TestSignXML(unittest.TestCase):
                 with self.assertRaises((InvalidInput, etree.XMLSyntaxError)):
                     XMLVerifier().verify(fh.read())
 
-    def test_soap_request_with_inclusive_namespaces(self):
-        with open(os.path.join(interop_dir, "soap", "request.xml")) as req_fh:
-            with self.assertRaises(etree.DocumentInvalid):
-                XMLVerifier().verify(req_fh.read(),
-                                     ca_pem_file=os.path.join(interop_dir, "soap", "ca.pem"),
-                                     expect_references=False)
-            req_fh.seek(0)
-            XMLVerifier().verify(req_fh.read(),
-                                 ca_pem_file=os.path.join(interop_dir, "soap", "ca.pem"),
-                                 expect_references=False,
-                                 validate_schema=False)
+    def test_signature_properties_with_detached_method(self):
+        doc = etree.Element('Test', attrib={'Id': 'mytest'})
+        sigprop = etree.Element('{http://somenamespace}MyCustomProperty')
+        sigprop.text = 'Some Text'
+        with open(os.path.join(os.path.dirname(__file__), "example.key"), "rb") as file:
+            key = file.read()
+        with open(os.path.join(os.path.dirname(__file__), "example.pem"), "rb") as file:
+            cert = file.read()
+        signature = XMLSigner(method=methods.detached).sign(doc,
+                                                            cert=cert,
+                                                            key=key,
+                                                            reference_uri="#mytest",
+                                                            signature_properties=sigprop)
+        fulldoc = b'<root>' + etree.tostring(signature) + etree.tostring(doc) + b'</root>'
+        XMLVerifier().verify(etree.fromstring(fulldoc), x509_cert=cert, expect_references=2)
+
+    def test_signature_properties_with_detached_method_re_enveloping(self):
+        doc = etree.Element('{http://somenamespace}Test', attrib={'Id': 'mytest'})
+        sigprop = etree.Element('{http://somenamespace}MyCustomProperty')
+        sigprop.text = 'Some Text'
+        with open(os.path.join(os.path.dirname(__file__), "example.key"), "rb") as file:
+            key = file.read()
+        with open(os.path.join(os.path.dirname(__file__), "example.pem"), "rb") as file:
+            cert = file.read()
+        signer = XMLSigner(method=methods.detached,
+                           c14n_algorithm="http://www.w3.org/TR/2001/REC-xml-c14n-20010315")
+        signer.namespaces['ns0'] = 'http://enveloping.namespace'
+        signature = signer.sign(doc,
+                                cert=cert,
+                                key=key,
+                                reference_uri="#mytest",
+                                signature_properties=sigprop)
+        fulldoc = b'<ns0:root xmlns:ns0="http://enveloping.namespace">' + etree.tostring(signature) + etree.tostring(doc) + b'</ns0:root>'
+        XMLVerifier().verify(etree.fromstring(fulldoc), x509_cert=cert, expect_references=2)
+
 
 if __name__ == '__main__':
     unittest.main()
